/**
 * Data structure containing document dates organized by year and month
 * Format: { year: { month: [days] } }
 * Example: { 2024: { 11: ["08"] } }
 */
const documentsByDate = {
  2025: {
    2: ["06"], // February - 1 event
  },
  2024: {
    11: ["08"], // November - 1 event
    10: ["03", "03"], // October - 2 events
    4: ["26", "25", "25"], // April - 3 events
    3: ["28", "11"], // March - 2 events
    2: ["29", "15", "08", "08", "06"], // February - 5 events
    1: ["26", "10"], // January - 2 events
    0: ["00"], // Unknown month - 1 event
  },
  2023: {
    11: ["19", "17"], // November - 2 events
    10: ["25", "19", "18", "09", "03", "03"], // October - 6 events
    9: ["26", "19", "18", "14", "11", "08", "08", "05", "05", "05", "01", "01"], // September - 12 events
    8: ["31", "14", "09"], // August - 3 events
    7: ["13"], // July - 1 event
    6: ["30"], // June - 1 event
    5: ["10"], // May - 1 event
    4: ["17"], // April - 1 event
    3: ["03"], // March - 1 event
  },
  2022: {
    12: ["14"], // December - 1 event
    11: ["08"], // November - 1 event
    9: ["20"], // September - 1 event
    8: ["26"], // August - 1 event
    3: ["07"], // March - 1 event
  },
  2019: {
    11: ["21"], // November - 1 event
    8: [""], // August - 1 event
    5: [""], // May - 1 event
  },
};

/**
 * Converts month name to its corresponding number
 * @param {string} monthName - Full name of the month (e.g., "January")
 * @returns {number} Month number (1-12)
 */
function getMonthNumber(monthName) {
  const months = {
    January: 1,
    February: 2,
    March: 3,
    April: 4,
    May: 5,
    June: 6,
    July: 7,
    August: 8,
    September: 9,
    October: 10,
    November: 11,
    December: 12,
  };
  return months[monthName];
}

/**
 * Counts total number of unique documents for a given year
 * @param {number} year - The year to count documents for
 * @returns {number} Total count of unique documents
 */
function countDocumentsForYear(year) {
  if (!documentsByDate[year]) return 0;

  // Count unique dates in each month
  return Object.values(documentsByDate[year]).reduce((total, monthDocs) => {
    // Use Set to count unique dates
    const uniqueDates = new Set(monthDocs);
    return total + uniqueDates.size;
  }, 0);
}

/**
 * Initializes the calendar interface and sets up event handlers
 * Manages the calendar view, month preview, and date preview modals
 */
async function initCalendar() {
  try {
    document.body.classList.add("loading");
    const yearButtons = document.querySelectorAll(".year-btn");
    const monthPreviewModal = document.getElementById("monthPreviewModal");
    const datePreviewModal = document.getElementById("datePreviewModal");
    const monthsGrid = document.getElementById("monthsGrid");
    const datesContainer = document.getElementById("datesContainer");
    const selectedYear = document.getElementById("selectedYear");
    const selectedMonth = document.getElementById("selectedMonth");
    const selectedYearInMonth = document.getElementById("selectedYearInMonth");
    const backToMonths = document.getElementById("backToMonths");

    // Add back button handler inside initCalendar
    backToMonths.addEventListener("click", function (e) {
      e.preventDefault();
      e.stopPropagation();
      datePreviewModal.style.display = "none";
      monthPreviewModal.style.display = "flex";
    });

    // Year button click handler
    yearButtons.forEach((btn) => {
      const year = btn.dataset.year;
      // Add has-events class if year has events
      if (documentsByDate[year]) {
        btn.classList.add("has-documents");
        // Add event count to tooltip
        const eventCount = countDocumentsForYear(year);
        btn.setAttribute(
          "title",
          `${eventCount} document${eventCount !== 1 ? "s" : ""}`
        );
      }

      btn.addEventListener("click", () => {
        showMonthsPreview(year);
      });
    });

    function showMonthsPreview(year) {
      const yearStr = year.toString();
      selectedYear.textContent = year;
      monthsGrid.innerHTML = "";

      // Create container for months
      const monthsContainer = document.createElement("div");
      monthsContainer.className = "months-container";

      // Add Unknown month at the beginning
      const unknownCard = createMonthCard("Unknown", "Unk", yearStr, "0");
      monthsGrid.appendChild(unknownCard);

      // Add regular months
      const months = [
        { name: "January", abbr: "Jan" },
        { name: "February", abbr: "Feb" },
        { name: "March", abbr: "Mar" },
        { name: "April", abbr: "Apr" },
        { name: "May", abbr: "May" },
        { name: "June", abbr: "Jun" },
        { name: "July", abbr: "Jul" },
        { name: "August", abbr: "Aug" },
        { name: "September", abbr: "Sep" },
        { name: "October", abbr: "Oct" },
        { name: "November", abbr: "Nov" },
        { name: "December", abbr: "Dec" },
      ];

      months.forEach((month, index) => {
        const monthCard = createMonthCard(
          month.name,
          month.abbr,
          yearStr,
          (index + 1).toString()
        );
        monthsGrid.appendChild(monthCard);
      });

      monthPreviewModal.style.display = "flex";

      // Add entrance animation
      setTimeout(() => {
        document.querySelectorAll(".month-card").forEach((card, index) => {
          card.style.animation = `fadeInUp 0.3s ease forwards ${index * 0.05}s`;
        });
      }, 100);
    }

    /**
     * Creates a month card element for the calendar view
     * @param {string} monthName - Full name of the month
     * @param {string} monthAbbr - Three-letter abbreviation of the month
     * @param {string} year - The year
     * @param {string} monthNum - Month number (0-11)
     * @returns {HTMLElement} The created month card element
     */
    function createMonthCard(monthName, monthAbbr, year, monthNum) {
      const monthCard = document.createElement("div");
      monthCard.className = "month-card";

      monthCard.innerHTML = `
        <div class="month-name">
          <span>${monthName}</span>
          <span class="month-abbr">${monthAbbr}</span>
        </div>
      `;

      // Check if month has events and add count
      const monthDocs = documentsByDate[year]?.[monthNum];
      if (monthDocs?.length) {
        const eventCount = monthDocs.length;
        monthCard.classList.add("has-documents");
        monthCard.setAttribute("data-event-count", eventCount);
        monthCard.setAttribute(
          "title",
          `${eventCount} document${eventCount > 1 ? "s" : ""}`
        );
      }

      monthCard.addEventListener("click", () => {
        showDatePreview(year, monthNum, monthName);
      });

      return monthCard;
    }

    /**
     * Displays the preview modal for a specific date
     * @param {string} year - The year
     * @param {string} month - The month number
     * @param {string} monthName - Name of the month
     */
    function showDatePreview(year, month, monthName) {
      selectedMonth.textContent = monthName;
      selectedYearInMonth.textContent = year;
      datesContainer.innerHTML = "";

      const monthStr = month.toString();
      const yearStr = year.toString();

      const monthDocs = documentsByDate[yearStr]?.[monthStr];
      if (monthDocs?.length) {
        const monthDocuments = [...new Set(monthDocs)].sort(
          (a, b) => parseInt(a) - parseInt(b)
        );

        const eventsContainer = document.createElement("div");
        eventsContainer.className = "events-container";

        monthDocuments.forEach((day) => {
          const eventDetails = findEventDetails(day, monthStr, yearStr);
          if (eventDetails) {
            eventDetails.forEach((event) => {
              const eventItem = document.createElement("div");
              eventItem.className = "event-item";
              eventItem.innerHTML = `
                <div class="event-header">
                  <div class="event-date">${day} ${monthName}</div>
                </div>
                <div class="event-content">
                  <div class="event-title">${event.title}</div>
                  <div class="event-source">${event.source}</div>
                </div>
              `;
              eventsContainer.appendChild(eventItem);
            });
          }
        });

        datesContainer.appendChild(eventsContainer);
      } else {
        const noEvents = document.createElement("div");
        noEvents.className = "no-documents";
        noEvents.textContent = "No documents in this month";
        datesContainer.appendChild(noEvents);
      }

      monthPreviewModal.style.display = "none";
      datePreviewModal.style.display = "flex";

      // Ensure modal is properly sized and scrollable
      datePreviewModal.offsetHeight;
    }

    /**
     * Finds event details for a specific date
     * @param {string} day - Day of the month
     * @param {string} month - Month number
     * @param {string} year - Year
     * @returns {Array|null} Array of matching events or null if none found
     */
    function findEventDetails(day, month, year) {
      const eventElements = document.querySelectorAll(".timeline-item");
      // Ensure we're working with strings and pad with leading zeros
      const searchDay = day.toString().padStart(2, "0");
      const searchMonth = month.toString().padStart(2, "0");
      const searchYear = year.toString();

      // Find all matching events for this date
      const matchingEvents = [];

      for (const element of eventElements) {
        const dateText = element.querySelector(".event-date").textContent;
        const cleanDate = dateText
          .replace(/^(Published|Updated): /, "")
          .replace(/;.*$/, "")
          .replace(/,/g, "")
          .trim();

        try {
          let eventDay = "",
            eventMonth = "",
            eventYear = "";

          const parts = cleanDate.split(/[\s-/]+/).filter((part) => part);

          if (parts.length === 3) {
            if (parts[0].length === 4) {
              [eventYear, eventMonth, eventDay] = parts;
            } else if (isNaN(parseInt(parts[0]))) {
              eventMonth = getMonthNumber(parts[0]).toString();
              [, eventDay, eventYear] = parts;
            } else {
              [eventDay, , eventYear] = parts;
              eventMonth = getMonthNumber(parts[1]).toString();
            }

            eventDay = eventDay.padStart(2, "0");
            eventMonth = eventMonth.padStart(2, "0");

            if (
              eventDay === searchDay &&
              eventMonth === searchMonth &&
              eventYear === searchYear
            ) {
              matchingEvents.push({
                title: element.querySelector(".event-title").textContent.trim(),
                source: element
                  .querySelector(".event-source")
                  .textContent.trim(),
                link: element.querySelector("a").href,
              });
            }
          }
        } catch (error) {
          console.error("Error parsing date:", {
            dateText,
            error,
            cleanDate,
          });
          continue;
        }
      }
      return matchingEvents.length > 0 ? matchingEvents : null;
    }

    // Add close button handlers with proper modal content references
    const closeMonthBtn = document.getElementById("closeMonthView");
    const closeDateBtn = document.getElementById("closeDateView");
    const monthModalContent = monthPreviewModal.querySelector(".modal-content");
    const dateModalContent = datePreviewModal.querySelector(".modal-content");

    // Update close button handlers
    if (closeMonthBtn) {
      closeMonthBtn.addEventListener("click", (e) => {
        e.preventDefault();
        e.stopPropagation();
        monthPreviewModal.style.display = "none";
        // Reset animations
        document.querySelectorAll(".month-card").forEach((card) => {
          card.style.animation = "none";
        });
      });
    }

    if (closeDateBtn) {
      closeDateBtn.addEventListener("click", (e) => {
        e.preventDefault();
        e.stopPropagation();
        datePreviewModal.style.display = "none";
      });
    }

    // Prevent modal content clicks from closing the modal
    if (monthModalContent) {
      monthModalContent.addEventListener("click", (e) => {
        e.stopPropagation();
      });
    }

    if (dateModalContent) {
      dateModalContent.addEventListener("click", (e) => {
        e.stopPropagation();
      });
    }

    // Close on outside click
    monthPreviewModal.addEventListener("click", (e) => {
      if (e.target === monthPreviewModal) {
        monthPreviewModal.style.display = "none";
        // Reset animations
        document.querySelectorAll(".month-card").forEach((card) => {
          card.style.animation = "none";
        });
      }
    });

    datePreviewModal.addEventListener("click", (e) => {
      if (e.target === datePreviewModal) {
        datePreviewModal.style.display = "none";
      }
    });

    // Add keyboard escape key support
    document.addEventListener("keydown", (e) => {
      if (e.key === "Escape") {
        monthPreviewModal.style.display = "none";
        datePreviewModal.style.display = "none";
        // Reset animations
        document.querySelectorAll(".month-card").forEach((card) => {
          card.style.animation = "none";
        });
      }
    });

    // Add keyboard navigation
    initKeyboardNavigation();
  } finally {
    document.body.classList.remove("loading");
  }
}

/**
 * Initializes the preview modal functionality
 * Sets up event listeners for timeline items and modal interactions
 */
function initPreviewModal() {
  const previewModal = document.getElementById("previewModal");
  const previewTitle = document.getElementById("previewTitle");
  const previewDate = document.getElementById("previewDate");
  const previewSource = document.getElementById("previewSource");
  const previewLink = document.getElementById("previewLink");
  const closePreview = document.getElementById("closePreview");

  // Add click handler to all timeline items
  document.querySelectorAll(".timeline-item").forEach((item) => {
    const link = item.querySelector(".event-title");
    const originalHref = link.href;

    // Make the entire item clickable
    item.addEventListener("click", (e) => {
      e.preventDefault();
      e.stopPropagation();

      // Get item details
      const title = link.textContent.trim();
      const date = item.querySelector(".event-date").textContent;
      const source = item.querySelector(".event-source").textContent;

      // Update preview modal
      previewTitle.textContent = title;
      previewDate.textContent = date;
      previewSource.textContent = source;
      previewLink.href = originalHref;

      // Show modal
      previewModal.style.display = "flex";
      setTimeout(() => {
        previewModal.querySelector(".preview-content").style.transform =
          "translateY(0)";
      }, 10);
    });

    // Prevent link click from triggering twice
    link.addEventListener("click", (e) => {
      e.preventDefault();
      e.stopPropagation();
      item.click(); // Trigger the parent item click
    });
  });

  // Close modal handlers
  closePreview.addEventListener("click", closePreviewModal);
  previewModal.addEventListener("click", (e) => {
    if (e.target === previewModal) {
      closePreviewModal();
    }
  });

  function closePreviewModal() {
    previewModal.querySelector(".preview-content").style.transform =
      "translateY(20px)";
    setTimeout(() => {
      previewModal.style.display = "none";
    }, 300);
  }

  // Add keyboard support
  document.addEventListener("keydown", (e) => {
    if (e.key === "Escape" && previewModal.style.display === "flex") {
      closePreviewModal();
    }
  });
}

/**
 * Initializes collapsible year and month groups
 * Sets up expand/collapse functionality with animations
 */
function initCollapsibleGroups() {
  // Year groups
  document.querySelectorAll(".year-header").forEach((header) => {
    const yearGroup = header.parentElement;
    const yearEvents = yearGroup.querySelector(".year-events");
    const chevron = header.querySelector(".chevron-icon");

    // Set initial expanded state
    yearGroup.classList.remove("collapsed");
    yearEvents.style.maxHeight = yearEvents.scrollHeight + "px";
    yearEvents.style.overflow = "hidden";
    yearEvents.style.transition = "max-height 0.3s ease-in-out";
    chevron.style.transform = "rotate(0deg)";

    header.addEventListener("click", () => {
      const isCollapsed = yearGroup.classList.contains("collapsed");

      if (isCollapsed) {
        // Expand
        yearGroup.classList.remove("collapsed");
        yearEvents.style.maxHeight = yearEvents.scrollHeight + "px";
        chevron.style.transform = "rotate(0deg)";
      } else {
        // Collapse
        yearGroup.classList.add("collapsed");
        yearEvents.style.maxHeight = "0px";
        chevron.style.transform = "rotate(-90deg)";
      }
    });
  });

  // Month groups
  document.querySelectorAll(".month-label").forEach((label) => {
    const monthGroup = label.parentElement;
    const monthEvents = monthGroup.querySelector(".month-events");

    // Add chevron to month label
    label.innerHTML += `
      <button class="toggle-btn" aria-label="Toggle month content">
        <svg class="chevron-icon" xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round">
          <polyline points="6 9 12 15 18 9"></polyline>
        </svg>
      </button>
    `;

    const chevron = label.querySelector(".chevron-icon");

    // Set initial expanded state
    monthGroup.classList.remove("collapsed");
    monthEvents.style.visibility = "visible";
    monthEvents.style.height = "auto";
    monthEvents.style.opacity = "1";
    chevron.style.transform = "rotate(0deg)";

    label.addEventListener("click", () => {
      const isCollapsed = monthGroup.classList.contains("collapsed");

      if (isCollapsed) {
        // Expand
        monthGroup.classList.remove("collapsed");
        monthEvents.style.visibility = "visible";
        monthEvents.style.height = "auto";
        monthEvents.style.opacity = "1";
        chevron.style.transform = "rotate(0deg)";
      } else {
        // Collapse
        monthGroup.classList.add("collapsed");
        monthEvents.style.visibility = "hidden";
        monthEvents.style.height = "0";
        monthEvents.style.opacity = "0";
        chevron.style.transform = "rotate(-90deg)";
      }
    });
  });
}

/**
 * Initializes expand/collapse all functionality
 * Sets up event listeners for the expand all and collapse all buttons
 */
function initExpandCollapse() {
  const expandAllBtn = document.getElementById("expandAll");
  const collapseAllBtn = document.getElementById("collapseAll");

  expandAllBtn.addEventListener("click", () => {
    document.querySelectorAll(".year-group").forEach((yearGroup) => {
      const yearEvents = yearGroup.querySelector(".year-events");
      const chevron = yearGroup.querySelector(".chevron-icon");
      yearGroup.classList.remove("collapsed");
      yearEvents.style.maxHeight = yearEvents.scrollHeight + "px";
      chevron.style.transform = "rotate(0deg)";

      // Also expand all months within this year
      yearGroup.querySelectorAll(".month-group").forEach((monthGroup) => {
        const monthEvents = monthGroup.querySelector(".month-events");
        const monthChevron = monthGroup.querySelector(".chevron-icon");
        monthGroup.classList.remove("collapsed");
        monthEvents.style.visibility = "visible";
        monthEvents.style.height = "auto";
        monthEvents.style.opacity = "1";
        if (monthChevron) {
          monthChevron.style.transform = "rotate(0deg)";
        }
      });
    });
  });

  collapseAllBtn.addEventListener("click", () => {
    document.querySelectorAll(".year-group").forEach((yearGroup) => {
      const yearEvents = yearGroup.querySelector(".year-events");
      const chevron = yearGroup.querySelector(".chevron-icon");
      yearGroup.classList.add("collapsed");
      yearEvents.style.maxHeight = "0px";
      chevron.style.transform = "rotate(-90deg)";

      // Also collapse all months within this year
      yearGroup.querySelectorAll(".month-group").forEach((monthGroup) => {
        const monthEvents = monthGroup.querySelector(".month-events");
        const monthChevron = monthGroup.querySelector(".chevron-icon");
        monthGroup.classList.add("collapsed");
        monthEvents.style.visibility = "hidden";
        monthEvents.style.height = "0";
        monthEvents.style.opacity = "0";
        if (monthChevron) {
          monthChevron.style.transform = "rotate(-90deg)";
        }
      });
    });
  });
}

/**
 * Initializes the overview section with expand/collapse functionality
 */
function initOverviewSection() {
  const overviewSection = document.querySelector(".overview-section");
  const overviewHeader = overviewSection.querySelector(".overview-header");
  const overviewContent = overviewSection.querySelector(".overview-content");

  // Set initial state to expanded
  overviewSection.classList.add("expanded");
  overviewContent.style.maxHeight = overviewContent.scrollHeight + "px";

  overviewHeader.addEventListener("click", () => {
    const isExpanded = overviewSection.classList.contains("expanded");

    if (isExpanded) {
      // Collapse
      overviewSection.classList.remove("expanded");
      overviewContent.style.maxHeight = "0px";
    } else {
      // Expand
      overviewSection.classList.add("expanded");
      overviewContent.style.maxHeight = overviewContent.scrollHeight + "px";
    }
  });

  // Add keyboard support
  overviewHeader.addEventListener("keydown", (e) => {
    if (e.key === "Enter" || e.key === " ") {
      e.preventDefault();
      overviewHeader.click();
    }
  });
}

/**
 * Scrolls to a specific event in the timeline
 * @param {string} dateString - Date string identifier
 */
function scrollToEvent(dateString) {
  const targetElement = document.querySelector(`[data-date="${dateString}"]`);
  if (targetElement) {
    // Close any open modals
    document.querySelectorAll(".calendar-modal").forEach((modal) => {
      modal.style.display = "none";
    });

    // Scroll to the element
    targetElement.scrollIntoView({
      behavior: "smooth",
      block: "center",
    });

    // Add highlight effect
    targetElement.classList.add("highlight");
    setTimeout(() => {
      targetElement.classList.remove("highlight");
    }, 2000);
  }
}

/**
 * Initializes keyboard navigation for the calendar
 * Handles left/right arrow keys for year navigation
 */
function initKeyboardNavigation() {
  document.addEventListener("keydown", (e) => {
    if (e.key === "ArrowLeft" || e.key === "ArrowRight") {
      const currentYear = document.querySelector(".year-btn.active");
      const years = Array.from(document.querySelectorAll(".year-btn"));
      const currentIndex = years.indexOf(currentYear);

      if (e.key === "ArrowLeft" && currentIndex > 0) {
        years[currentIndex - 1].click();
      } else if (e.key === "ArrowRight" && currentIndex < years.length - 1) {
        years[currentIndex + 1].click();
      }
    }
  });
}

/**
 * Shows event popup for a specific date
 * @param {number} date - Day of the month
 * @param {number} year - Year
 * @param {number} month - Month number
 */
function showEventPopup(date, year, month) {
  console.log("showEventPopup called with:", { date, year, month });
  // Find the event details directly
  const eventDetails = findEventDetails(
    date.toString(),
    month.toString(),
    year.toString()
  );
  console.log("eventDetails found:", eventDetails);
  if (!eventDetails) return;

  // Use the existing preview modal
  const previewModal = document.getElementById("previewModal");
  const previewTitle = document.getElementById("previewTitle");
  const previewDate = document.getElementById("previewDate");
  const previewSource = document.getElementById("previewSource");
  const previewLink = document.getElementById("previewLink");

  // Update preview modal content
  previewTitle.textContent = eventDetails.title;
  previewDate.textContent = `${date} ${getMonthName(month)} ${year}`;
  previewSource.textContent = eventDetails.source;
  previewLink.href = eventDetails.link;

  // Show modal with animation
  previewModal.style.display = "flex";
  previewModal.querySelector(".preview-content").style.transform =
    "translateY(20px)";
  setTimeout(() => {
    previewModal.querySelector(".preview-content").style.transform =
      "translateY(0)";
  }, 10);

  // Close any open calendar modals
  document.querySelectorAll(".calendar-modal").forEach((modal) => {
    modal.style.display = "none";
  });
}

/**
 * Gets the full name of a month from its number
 * @param {number} monthNumber - Month number (1-12)
 * @returns {string} Full name of the month
 */
function getMonthName(monthNumber) {
  const months = [
    "January",
    "February",
    "March",
    "April",
    "May",
    "June",
    "July",
    "August",
    "September",
    "October",
    "November",
    "December",
  ];
  return months[parseInt(monthNumber) - 1];
}

/**
 * Updates the UK clock display with current time
 * Updates every second with formatted date and time
 */
function updateUKClock() {
  const now = new Date();

  const dateStr = now.toLocaleDateString("en-GB", {
    timeZone: "Europe/London",
    day: "2-digit",
    month: "long",
    year: "numeric",
  });

  const timeStr = now.toLocaleTimeString("en-GB", {
    timeZone: "Europe/London",
    hour12: false,
    hour: "2-digit",
    minute: "2-digit",
    second: "2-digit",
  });

  document.getElementById(
    "ukClock"
  ).textContent = `Real-time UK: ${dateStr} | ${timeStr}`;
}

/**
 * Initializes theme based on time of day
 * Uses sunrise-sunset API to determine appropriate theme
 */
async function initializeTheme() {
  // Get UK coordinates (London)
  const lat = 51.5074;
  const lng = -0.1278;

  try {
    const response = await fetch(
      `https://api.sunrise-sunset.org/json?lat=${lat}&lng=${lng}&formatted=0`
    );
    const data = await response.json();

    if (data.status === "OK") {
      const sunrise = new Date(data.results.sunrise);
      const sunset = new Date(data.results.sunset);
      const now = new Date();

      // Set initial theme based on time
      const isDaytime = now > sunrise && now < sunset;
      setTheme(isDaytime ? "light" : "dark");

      // Schedule next theme change
      const nextChange = isDaytime ? sunset : sunrise;
      const timeUntilChange = nextChange - now;

      if (timeUntilChange > 0) {
        setTimeout(() => {
          setTheme(isDaytime ? "dark" : "light");
          // Reinitialize for the next day
          initializeTheme();
        }, timeUntilChange);
      }
    }
  } catch (error) {
    console.error("Error fetching sunrise/sunset times:", error);
    // Use system preference as fallback
    const prefersDark = window.matchMedia(
      "(prefers-color-scheme: dark)"
    ).matches;
    setTheme(prefersDark ? "dark" : "light");
  }
}

/**
 * Sets the theme for the application
 * @param {string} theme - Theme name ('light' or 'dark')
 */
function setTheme(theme) {
  document.documentElement.setAttribute("data-theme", theme);
  localStorage.setItem("theme", theme);
  document
    .getElementById("themeToggle")
    .setAttribute("aria-checked", theme === "dark");
}

/**
 * Initializes theme toggle functionality
 * Sets up event listeners for theme switching
 */
function initThemeToggle() {
  const themeToggle = document.getElementById("themeToggle");
  const prefersDark = window.matchMedia("(prefers-color-scheme: dark)").matches;

  // Set initial theme based on system preference if no stored preference
  if (!localStorage.getItem("theme")) {
    setTheme(prefersDark ? "dark" : "light");
  }

  themeToggle.addEventListener("click", () => {
    const currentTheme = document.documentElement.getAttribute("data-theme");
    const newTheme = currentTheme === "light" ? "dark" : "light";
    setTheme(newTheme);

    // Update toggle button appearance and state
    const toggleCircle = themeToggle.querySelector(".toggle-circle");
    toggleCircle.style.transform =
      newTheme === "dark" ? "translateX(32px)" : "translateX(0)";
    themeToggle.setAttribute("aria-checked", newTheme === "dark");
  });
}

/**
 * Cleans up event listeners and intervals
 * Called before page unload
 */
function cleanup() {
  window.removeEventListener("scroll", handleScroll);
  document.removeEventListener("keydown", handleKeydown);
  clearInterval(ukClockInterval);
}

/**
 * Shows loading spinner
 */
function showLoading() {
  const loading = document.createElement("div");
  loading.className = "loading";
  loading.innerHTML = '<div class="loading-spinner"></div>';
  document.body.appendChild(loading);
}

/**
 * Hides loading spinner
 */
function hideLoading() {
  const loading = document.querySelector(".loading");
  if (loading) {
    loading.remove();
  }
}

/**
 * Loads content with loading indicator
 */
async function loadContent() {
  showLoading();
  try {
    // Async operations
  } catch (error) {
    console.error("Error loading content:", error);
  } finally {
    hideLoading();
  }
}

/**
 * Closes a modal with animation
 * @param {HTMLElement} modalElement - The modal to close
 */
function closeModal(modalElement) {
  if (!modalElement) return;

  const content = modalElement.querySelector(
    ".modal-content, .preview-content"
  );
  if (content) {
    content.style.transform = "translateY(20px)";
  }

  setTimeout(() => {
    modalElement.style.display = "none";
    document.body.style.overflow = "";
  }, 300);
}

// Initialize calendar
document.addEventListener("DOMContentLoaded", () => {
  initCalendar();
  initPreviewModal();
  initCollapsibleGroups();
  initExpandCollapse();
  initOverviewSection();

  // Add click handler for dates
  document.getElementById("datesContainer").addEventListener("click", (e) => {
    const dateCell = e.target.closest(".date-cell");
    if (!dateCell || !dateCell.classList.contains("has-document")) return;

    const date = parseInt(dateCell.textContent);
    const year = parseInt(
      document.getElementById("selectedYearInMonth").textContent
    );
    const month = getMonthNumber(
      document.getElementById("selectedMonth").textContent
    );

    showEventPopup(date, year, month);
  });
});

// Add CSS animation for fade in effect
const style = document.createElement("style");
style.textContent = `
  @keyframes fadeInUp {
    from {
      opacity: 0;
      transform: translateY(20px);
    }
    to {
      opacity: 1;
      transform: translateY(0);
    }
  }
`;
document.head.appendChild(style);

// Initialize theme when the page loads
document.addEventListener("DOMContentLoaded", () => {
  initThemeToggle();
  initializeTheme(); // Start automatic theme switching
});

// Scroll to top functionality
document.addEventListener("DOMContentLoaded", function () {
  const scrollToTopBtn = document.getElementById("scrollToTop");

  // Show button when page is scrolled
  window.addEventListener("scroll", function () {
    if (window.pageYOffset > 300) {
      scrollToTopBtn.classList.add("visible");
    } else {
      scrollToTopBtn.classList.remove("visible");
    }
  });

  // Smooth scroll to top when button is clicked
  scrollToTopBtn.addEventListener("click", function () {
    window.scrollTo({
      top: 0,
      behavior: "smooth",
    });
  });
});

// Event handler variables
let handleScroll, handleKeydown;

document.addEventListener("DOMContentLoaded", () => {
  // Define reusable event handlers
  handleScroll = () => {
    const scrollToTopBtn = document.getElementById("scrollToTop");
    if (window.pageYOffset > 300) {
      scrollToTopBtn.classList.add("visible");
    } else {
      scrollToTopBtn.classList.remove("visible");
    }
  };

  handleKeydown = (e) => {
    if (e.key === "Escape") {
      const modals = document.querySelectorAll(
        ".calendar-modal, .preview-modal"
      );
      modals.forEach((modal) => {
        if (modal.style.display === "flex") {
          modal.style.display = "none";
          document.body.style.overflow = "";
        }
      });
    }
  };

  // Add event listeners
  window.addEventListener("scroll", handleScroll);
  document.addEventListener("keydown", handleKeydown);
});

// Modal close button handlers
document.querySelectorAll(".close-button").forEach((btn) => {
  btn.addEventListener("click", (e) => {
    e.preventDefault();
    const modal = btn.closest(".calendar-modal, .preview-modal");
    closeModal(modal);
  });
});

// Cleanup before window unload
window.addEventListener("beforeunload", cleanup);

// Initialize UK clock
let ukClockInterval = setInterval(updateUKClock, 1000);
updateUKClock(); // Initial call to display time immediately

// Initialize privacy policy and terms of use modals
document.addEventListener("DOMContentLoaded", function () {
  const privacyPolicyModal = document.getElementById("privacyPolicyModal");
  const privacyPolicyLink = document.getElementById("privacyPolicyLink");
  const closePrivacyPolicy = document.getElementById("closePrivacyPolicy");

  // Open privacy policy modal
  privacyPolicyLink.addEventListener("click", function (e) {
    e.preventDefault();
    privacyPolicyModal.style.display = "flex";
    document.body.style.overflow = "hidden"; // Prevent scrolling when modal is open
  });

  // Close privacy policy modal
  closePrivacyPolicy.addEventListener("click", function () {
    privacyPolicyModal.style.display = "none";
    document.body.style.overflow = ""; // Restore scrolling
  });

  // Close modal when clicking outside
  privacyPolicyModal.addEventListener("click", function (e) {
    if (e.target === privacyPolicyModal) {
      privacyPolicyModal.style.display = "none";
      document.body.style.overflow = "";
    }
  });

  // Close modal on escape key
  document.addEventListener("keydown", function (e) {
    if (e.key === "Escape" && privacyPolicyModal.style.display === "flex") {
      privacyPolicyModal.style.display = "none";
      document.body.style.overflow = "";
    }
  });

  // Terms of Use Modal
  const termsOfUseModal = document.getElementById("termsOfUseModal");
  const termsOfUseLink = document.getElementById("termsOfUseLink");
  const closeTermsOfUse = document.getElementById("closeTermsOfUse");

  // Open terms of use modal
  termsOfUseLink.addEventListener("click", function (e) {
    e.preventDefault();
    termsOfUseModal.style.display = "flex";
    document.body.style.overflow = "hidden"; // Prevent scrolling when modal is open
  });

  // Close terms of use modal
  closeTermsOfUse.addEventListener("click", function () {
    termsOfUseModal.style.display = "none";
    document.body.style.overflow = ""; // Restore scrolling
  });

  // Close modal when clicking outside
  termsOfUseModal.addEventListener("click", function (e) {
    if (e.target === termsOfUseModal) {
      termsOfUseModal.style.display = "none";
      document.body.style.overflow = "";
    }
  });

  // Close modal on escape key (update existing handler)
  document.addEventListener("keydown", function (e) {
    if (e.key === "Escape") {
      if (termsOfUseModal.style.display === "flex") {
        termsOfUseModal.style.display = "none";
        document.body.style.overflow = "";
      }
      if (privacyPolicyModal.style.display === "flex") {
        privacyPolicyModal.style.display = "none";
        document.body.style.overflow = "";
      }
    }
  });
});
